"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _instrumentationHelpers = require("../utils/instrumentation-helpers.js");
var _InstrumentationModule = _interopRequireDefault(require("../../native/InstrumentationModule.js"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function sanitizeCustomData(object) {
  switch (typeof object) {
    case 'symbol':
      {
        return object.description;
      }
    case 'function':
      {
        // Remove function body
        const split = object.toString().split('{');
        const [functionName] = split;
        return functionName.slice(0, -1);
      }
    default:
      {
        return `${object}`;
      }
  }
}

/**
 * Helps tracking functions and their implementation.
 *
 * The {@link InfoPoint} decorator provides an easier way to add info points to your app.
 * Consider using that as a first resort.
 *
 * ```tsx
 * async function downloadImage(url) {
 *   return Instrumentation.trackCall('DownloadService', 'downloadImage', [url], async () => {
 *     const request = createRequest(url);
 *     const response = await request.execute();
 *     if (response.status >= 400) {
 *       throw new Error(`Bad status code: ${response.status}`);
 *     }
 *   return response.data;
 *  });
 * ```
 *
 * @param {string} className Name of the class where the function is called.
 * @param {string} functionName Name of the class where the function is called.
 * @param {any[]} args Function's arguments.
 * @param {Function} executionCallback Function's implementation/body.
 */
async function trackCall(className, functionName, args, executionCallback) {
  const id = await _InstrumentationModule.default.beginCall(className, functionName, args.map(element => sanitizeCustomData(element)), false);
  try {
    const ret = executionCallback();
    if ((0, _instrumentationHelpers.isPromise)(ret)) {
      return ret.then(onSuccess, onFailure);
    }
    return onSuccess(ret);
  } catch (error) {
    return onFailure(error);
  }
  async function onSuccess(result) {
    await _InstrumentationModule.default.endCallWithSuccess(id, {
      result: sanitizeCustomData(result)
    });
    return result;
  }
  async function onFailure(error) {
    await _InstrumentationModule.default.endCallWithError(id, (0, _instrumentationHelpers.convertToError)(error));
    throw error;
  }
}
var _default = exports.default = trackCall;
//# sourceMappingURL=trackCall.js.map