"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ScreenTracker = void 0;
var _InstrumentationModule = _interopRequireDefault(require("../../native/InstrumentationModule.js"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// eslint-disable-next-line @typescript-eslint/no-unused-vars

/**
 * WARNING: Planned deprecation.
 * Migrate to {@linkcode useScreenTracker} instead.
 * ---
 * Class used for manually tracking components throughout the app.
 *
 * The available methods permit specifying the start and end of a screen.
 * This will be reflected as an app screen in the controller.
 *
 * Some navigation libraries permit subscribing to root level methods where
 * you only have to implement this API only once, instead of in every screen:
 *
 * 1. {@link https://wix.github.io/react-native-navigation/docs/screen-lifecycle React Native navigation}
 * `componentDidAppear()` & `componentDidDisappear()`.
 *
 * 2. {@link https://reactnavigation.org/docs/screen-trackingReact React navigation}
 * `onStateChange()`.
 *
 * WARNING: Use unique screen names. Duplicate names might result in unexpected
 * behavior.
 *
 * ```tsx
 * import { ScreenTracker, useScreenTracker } from '@appdynamics/react-native-agent';
 *
 * class LoginScreen extends Component<{}> {
 *   async componentDidMount() {
 *     await ScreenTracker.getInstance().trackScreenStart('LoginScreen');
 *   }
 *
 *   async componentWillUnmount() {
 *     await ScreenTracker.getInstance().trackScreenEnd('LoginScreen');
 *   }
 *
 *   public render() {
 *     return (
 *       <View>
 *         <Text>My login screen.</Text>
 *       </View>
 *     );
 *   }
 * }
 * ```
 */
class ScreenTracker {
  // eslint-disable-next-line no-useless-constructor, no-empty-function
  constructor() {}
  static getInstance() {
    return this.instance ||= new this();
  }
  trackedScreens = {};
  trackScreenStart = async screenName => {
    const startDate = Date.now();
    const partialScreen = {
      screenName,
      startDate
    };
    const uuidString = await _InstrumentationModule.default.trackScreenStart(partialScreen);
    const trackedScreen = {
      uuidString,
      ...partialScreen
    };
    this.trackedScreens[trackedScreen.screenName] = trackedScreen;
  };
  trackScreenEnd = async screenName => {
    if (screenName in this.trackedScreens) {
      const trackedScreen = this.trackedScreens[screenName];
      const endDate = Date.now();
      const finalScreen = {
        endDate,
        ...trackedScreen
      };
      await _InstrumentationModule.default.trackScreenEnd(finalScreen);
      delete this.trackedScreens[screenName];
    }
  };
}
exports.ScreenTracker = ScreenTracker;
//# sourceMappingURL=ScreenTracker.old.js.map