"use strict";

import InstrumentationModule from "../../native/InstrumentationModule.js";

/**
 * Reports metric value for the given name.
 *
 * The name should contain only alphanumeric characters and spaces. Illegal
 * characters will be replaced by their ASCII hex value.
 *
 * **WARNING:** The native agent expects a 64-bit signed integer. However,
 * JavaScript numbers are never represented as such.
 * The maximum and minimum integers which can be represented with exact accuracy
 * are materialized by {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER MAX_SAFE_INTEGER}
 * and {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_SAFE_INTEGER MIN_SAFE_INTEGER}.
 *
 * Any value that falls outside of this range will be reported with a different
 * metric name, containing the word "INVALID".
 *
 * @param {string} name The name of the metric key.
 * @param {number} value The value reported for the given key. Must be an
 * integer between {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER MAX_SAFE_INTEGER}
 * and
 * {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MIN_SAFE_INTEGER MIN_SAFE_INTEGER}.
 */
async function reportMetric(name, value) {
  let finalName = name;
  let finalValue = value;
  if (!Number.isInteger(value)) {
    finalName = `${name} INVALID not an integer`;
    finalValue = 0;
  }
  if (!Number.isInteger(value)) {
    finalName = `${name} INVALID not an integer`;
    finalValue = 0;
  }
  if (value > Number.MAX_SAFE_INTEGER) {
    finalName = `${name} INVALID value too high`;
    finalValue = Number.MAX_SAFE_INTEGER;
  } else if (value < Number.MIN_SAFE_INTEGER) {
    finalName = `${name} INVALID value too low`;
    finalValue = Number.MIN_SAFE_INTEGER;
  }
  await InstrumentationModule.reportMetric(finalName, finalValue);
}
export default reportMetric;
//# sourceMappingURL=reportMetric.js.map