"use strict";

import React, { useRef, useCallback, createContext } from 'react';
import InstrumentationModule from "../../native/InstrumentationModule.js";
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { jsx as _jsx } from "react/jsx-runtime";
const ScreenTrackerContext = /*#__PURE__*/createContext(null);
/**
 * Context that provides access to the {@linkcode useScreenTracker} hook.
 *
 * Wrap the topmost component that will consume `useScreenTracker`, ideally on
 * top of your `App` component:
 *
 * ```tsx
 * export default function App() {
 *
 *  // misc implementation
 *
 *  return (
 *    <ScreenTrackerProvider>
 *      <View>
 *        // rest of JSX tree
 *      </View>
 *    </ScreenTrackerProvider>
 *  );
 *  ```
 *
 * @param {React.ReactNode} props Your component's props.
 * @returns {React.ReactNode} Wrapper around your component.
 */
const ScreenTrackerProvider = props => {
  const trackedScreens = useRef({});
  const trackScreenStart = useCallback(async screenName => {
    const startDate = Date.now();
    const uuidString = await InstrumentationModule.trackScreenStart({
      screenName,
      startDate
    });
    trackedScreens.current[screenName] = {
      screenName,
      startDate,
      uuidString
    };
  }, []);
  const trackScreenEnd = useCallback(async screenName => {
    if (screenName in trackedScreens.current) {
      const trackedScreen = trackedScreens.current[screenName];
      const endDate = Date.now();
      const finalScreen = {
        endDate,
        ...trackedScreen
      };
      await InstrumentationModule.trackScreenEnd(finalScreen);
      delete trackedScreens.current[screenName];
    }
  }, []);
  return /*#__PURE__*/_jsx(ScreenTrackerContext.Provider, {
    value: {
      trackScreenStart,
      trackScreenEnd
    },
    ...props
  });
};
export { ScreenTrackerProvider, ScreenTrackerContext };
//# sourceMappingURL=ScreenTrackerProvider.js.map