type RequestTrackerHeaders = {
    [key: string]: string;
};
/**
 * Create and configure a RequestTracker object before sending an HTTP request to track it manually.
 *
 * If the SDK does not automatically discover your HTTP requests, use this class to explicitly
 * report them. Note that most users will not need to use this class; check the documentation
 * for the list of HTTP APIs that are automatically discovered.
 *
 *  Warning: `Instrumentation.start({ appKey: <YOUR_APP_KEY>})` must've been called.
 *
 * Usage:
 *
 * ```ts
 *  import {RequestTracker} from '@appdynamics/react-native-agent';
 *
 *  function sendRequest() {
 *    const tracker = await RequestTracker("https://www.appdynamics.com");
 *
 *    const correlationHeaders = await RequestTracker.getServerCorrelationHeaders();
 *
 *    const headers = {
 *      'Custom-Header': 'Test-Header',
 *       ...correlationHeaders
 *    };
 *    await tracker.setRequestHeaders(headers);
 *
 *    try {
 *      const response = await fetch(url);
 *
 *      const responseHeaders = {};
 *      response.headers.forEach((value: string, key: string) => {
 *        responseHeaders[key] = value;
 *      });
 *
 *      await tracker.setResponseStatusCode(response.status);
 *      await tracker.setResponseHeaders(responseHeaders);
 *      await tracker.setUserDataDate('timestamp', new Date());
 *    } catch (error: any) {
 *      await tracker.setError(error);
 *    } finally {
 *      await tracker.reportDone();
 *    }
 * }
 * ```
 *
 * @param {string} url The URL being requested.
 * @returns {object} Tracker object.
 */
declare function RequestTracker(url: string): Promise<{
    setError: (error: Error) => Promise<void>;
    setRequestHeaders: (requestHeaders: RequestTrackerHeaders) => Promise<void>;
    setResponseHeaders: (responseHeaders: RequestTrackerHeaders) => Promise<void>;
    setUserData: (key: string, value: string) => Promise<void>;
    setUserDataBoolean: (key: string, value: boolean) => Promise<void>;
    setUserDataNumber: (key: string, value: number) => Promise<void>;
    setUserDataDate: (key: string, value: Date) => Promise<void>;
    setResponseStatusCode: (statusCode: number) => Promise<void>;
    reportDone: () => Promise<void>;
}>;
export { RequestTracker, type RequestTrackerHeaders };
//# sourceMappingURL=RequestTracker.d.ts.map