import InstrumentationModule from '../../native/InstrumentationModule';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import type useScreenTracker from '../functions/useScreenTracker';

export type TrackedScreen = {
  screenName: string;
  startDate: number;
  uuidString: string;
};

/**
 * WARNING: Planned deprecation.
 * Migrate to {@linkcode useScreenTracker} instead.
 * ---
 * Class used for manually tracking components throughout the app.
 *
 * The available methods permit specifying the start and end of a screen.
 * This will be reflected as an app screen in the controller.
 *
 * Some navigation libraries permit subscribing to root level methods where
 * you only have to implement this API only once, instead of in every screen:
 *
 * 1. {@link https://wix.github.io/react-native-navigation/docs/screen-lifecycle React Native navigation}
 * `componentDidAppear()` & `componentDidDisappear()`.
 *
 * 2. {@link https://reactnavigation.org/docs/screen-trackingReact React navigation}
 * `onStateChange()`.
 *
 * WARNING: Use unique screen names. Duplicate names might result in unexpected
 * behavior.
 *
 * ```tsx
 * import { ScreenTracker, useScreenTracker } from '@appdynamics/react-native-agent';
 *
 * class LoginScreen extends Component<{}> {
 *   async componentDidMount() {
 *     await ScreenTracker.getInstance().trackScreenStart('LoginScreen');
 *   }
 *
 *   async componentWillUnmount() {
 *     await ScreenTracker.getInstance().trackScreenEnd('LoginScreen');
 *   }
 *
 *   public render() {
 *     return (
 *       <View>
 *         <Text>My login screen.</Text>
 *       </View>
 *     );
 *   }
 * }
 * ```
 */
export class ScreenTracker {
  private static instance: ScreenTracker;

  // eslint-disable-next-line no-useless-constructor, no-empty-function
  private constructor() {}

  public static getInstance(): ScreenTracker {
    return (this.instance ||= new this());
  }

  private trackedScreens: Record<string, TrackedScreen> = {};

  trackScreenStart = async (screenName: string) => {
    const startDate = Date.now();
    const partialScreen: Omit<TrackedScreen, 'uuidString'> = {
      screenName,
      startDate,
    };

    const uuidString = await InstrumentationModule.trackScreenStart(
      partialScreen,
    );
    const trackedScreen: TrackedScreen = {
      uuidString,
      ...partialScreen,
    };

    this.trackedScreens[trackedScreen.screenName] = trackedScreen;
  };

  trackScreenEnd = async (screenName: string) => {
    if (screenName in this.trackedScreens) {
      const trackedScreen = this.trackedScreens[screenName]!;
      const endDate = Date.now();
      const finalScreen: TrackedScreen & {endDate: number} = {
        endDate,
        ...trackedScreen,
      };

      await InstrumentationModule.trackScreenEnd(finalScreen);
      delete this.trackedScreens[screenName];
    }
  };
}
