import React, {useRef, useCallback, createContext} from 'react';
import InstrumentationModule from '../../native/InstrumentationModule';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import type useScreenTracker from '../functions/useScreenTracker';

interface ScreenTrackerContextProps {
  readonly trackScreenStart: (screenName: string) => void;
  readonly trackScreenEnd: (screenName: string) => void;
}

const ScreenTrackerContext = createContext<ScreenTrackerContextProps | null>(
  null,
);

type TrackedScreen = {
  screenName: string;
  startDate: number;
  uuidString: string;
};

/**
 * Context that provides access to the {@linkcode useScreenTracker} hook.
 *
 * Wrap the topmost component that will consume `useScreenTracker`, ideally on
 * top of your `App` component:
 *
 * ```tsx
 * export default function App() {
 *
 *  // misc implementation
 *
 *  return (
 *    <ScreenTrackerProvider>
 *      <View>
 *        // rest of JSX tree
 *      </View>
 *    </ScreenTrackerProvider>
 *  );
 *  ```
 *
 * @param {React.ReactNode} props Your component's props.
 * @returns {React.ReactNode} Wrapper around your component.
 */
const ScreenTrackerProvider = (props: any) => {
  const trackedScreens = useRef<Record<string, TrackedScreen>>({});

  const trackScreenStart = useCallback(async (screenName: string) => {
    const startDate = Date.now();
    const uuidString = await InstrumentationModule.trackScreenStart({
      screenName,
      startDate,
    });

    trackedScreens.current[screenName] = {
      screenName,
      startDate,
      uuidString,
    };
  }, []);

  const trackScreenEnd = useCallback(async (screenName: string) => {
    if (screenName in trackedScreens.current) {
      const trackedScreen = trackedScreens.current[screenName]!;

      const endDate = Date.now();
      const finalScreen: TrackedScreen & {endDate: number} = {
        endDate,
        ...trackedScreen,
      };

      await InstrumentationModule.trackScreenEnd(finalScreen);
      delete trackedScreens.current[screenName];
    }
  }, []);

  return (
    <ScreenTrackerContext.Provider
      value={{
        trackScreenStart,
        trackScreenEnd,
      }}
      {...props}
    />
  );
};

export {
  type TrackedScreen,
  ScreenTrackerProvider,
  type ScreenTrackerContextProps,
  ScreenTrackerContext,
};
